<?php
/**
 *  
 *  Overrides (Drupal-ifies) functions in Viele's download.php
 *
 *  This class overrides Downloader::processBatchResponse and
 *  Downloader::write_rdb in order to make the Viele download
 *  handler compatible with Drupal node handling.
 *
 *  Copyright 2008, Arthur Richards awjrichards@gmail.com
 *  
 *  This file is part of the MLS module for Drupal.
 *
 *  Foobar is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  Foobar is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with MLS module.  If not, see <http://www.gnu.org/licenses/>.
 * 
 */
/**
 * Overrides (Drupal-ifies) functions in Viele's download.php
 *
 * This class overrides Downloader::processBatchResponse and
 * Downloader::write_rdb in order to make the Viele download
 * handler compatible with Drupal node handling.
 */
class Downloader_Drupal extends Downloader {
  private $mls = NULL;
  private $user = NULL;
  private $existing_listings = array();
  private $new_listings = array();
  
  function __construct($mls,$user){
  	$this->mls = $mls;
    $this->user = $user;
  }

  function get_mls(){
  	return $this->mls;
  }

  function get_user(){
    return $this->user;
  } 
  
  /**
	 * TODO: refractor tihs badboy.
	 *
	 * @param unknown_type $quiet
	 * @param unknown_type $batch_number
	 * @param unknown_type $DATA_RESPONSE
	 * @param unknown_type $SOURCE_CONTEXT
	 * @param unknown_type $TARGET_CONTEXT
	 * @param unknown_type $EXTRACT_CONTEXT
	 * @param unknown_type $HTTP_CONTEXT
	 * @param unknown_type $METADATA_TABLE
	 * @param unknown_type $withDetail
	 * @return unknown
	 */
	function processBatchResponse($quiet,
		$batch_number,
		$DATA_RESPONSE,
		$SOURCE_CONTEXT,
		$TARGET_CONTEXT,
		$EXTRACT_CONTEXT,
		$HTTP_CONTEXT,
		$METADATA_TABLE,
		$withDetail) {

		$BATCH_RESULT = new Statistics('Batch #' . $batch_number);
		if ($EXTRACT_CONTEXT->trace) {
			if ($withDetail) {
				$BATCH_RESULT->printBatchStart();
			}
		}

		// if RDB-based

		switch ($TARGET_CONTEXT->target_type) {
			case 'RDB':
				//drupal already has us connected to the DB
				//$conn = $this->createConnection($TARGET_CONTEXT);
				break;
			default:
				drupal_set_message(t('Viele target type is unsupported - you may only use a relational database target.'),'error');
				return '';
		}

		// initialize

		$unique_key = $DATA_RESPONSE->getUniqueKey();
		$ownership_field = $SOURCE_CONTEXT->ownership;
		$types = $DATA_RESPONSE->getTypes();
		$interpretations = $DATA_RESPONSE->getInterpretations();

		// create listing objects

		$rowCount = $DATA_RESPONSE->getRowCount();
		for ($i = 0; $i < $rowCount; ++$i) {
			$data_row = $DATA_RESPONSE->getRow($i);
			$LISTING_OBJECT = new ListingObject();

			if ($EXTRACT_CONTEXT->image_maps == NULL) {
				if ($EXTRACT_CONTEXT->data_maps == NULL) {
					$tempMap = explode(',', $EXTRACT_CONTEXT->extract_column_list);
					$LISTING_OBJECT->setData($tempMap,
						$data_row,
						$types,
						$interpretations,
						$unique_key,
						$tempMap,
						$EXTRACT_CONTEXT->metacolumn_map);
				}else {
					if (array_key_exists('SOURCE', $EXTRACT_CONTEXT->data_maps)) {
						$LISTING_OBJECT->setData($EXTRACT_CONTEXT->data_maps['SOURCE'],
							$data_row,
							$types,
							$interpretations,
							$unique_key,
							$EXTRACT_CONTEXT->data_maps['TARGET'],
							$EXTRACT_CONTEXT->metacolumn_map);
					}else {
						$tempMap = explode(',', $EXTRACT_CONTEXT->extract_column_list);
						$LISTING_OBJECT->setData($tempMap,
							$data_row,
							$types,
							$interpretations,
							$unique_key,
							$tempMap,
							$EXTRACT_CONTEXT->metacolumn_map);
					}
				}
			}else {
				$LISTING_OBJECT->setData($EXTRACT_CONTEXT->data_maps['SOURCE'],
					$data_row,
					$types,
					$interpretations,
					$unique_key,
					$EXTRACT_CONTEXT->data_maps['TARGET'],
					$EXTRACT_CONTEXT->metacolumn_map,
					$EXTRACT_CONTEXT->image_maps['SOURCE'],
					$EXTRACT_CONTEXT->image_maps['TARGET']);
			}

			if (array_key_exists($ownership_field, $data_row)) {
				$LISTING_OBJECT->setOwner($data_row[$ownership_field]);
			}

			$unique_value = $LISTING_OBJECT->getUniqueValue();
			$buffer = ' UNKNOWN TARGET TYPE';

			// if the RETS Server supports location, use proxies

			$useImages = true;

			if ($useImages) {
				if ($SOURCE_CONTEXT->media_location) {
					if ($this->EXCHANGE->getTransportTrace()) {
						$this->EXCHANGE->trace('Downloading images via Media Server because the RETS server supports Location');
					}
					$this->bindImages($LISTING_OBJECT,
						$SOURCE_CONTEXT,
						$EXTRACT_CONTEXT,
						$HTTP_CONTEXT);
				}else {
					if ($this->EXCHANGE->getTransportTrace()) {
						$this->EXCHANGE->trace('Downloading images directly because the RETS server does not support Location');
					}
				}
			}
			switch ($TARGET_CONTEXT->target_type) {
				case 'RDB':
					$buffer = $this->write_rdb($LISTING_OBJECT,
						/*$conn,*/
						$SOURCE_CONTEXT,
						$TARGET_CONTEXT,
						$EXTRACT_CONTEXT,
						$BATCH_RESULT,
						$METADATA_TABLE,
						$unique_value);
					break;
				
				default:
					drupal_set_message(t('Viele target type is unsupported you may only use a relational database target.'),'error');
					break;
			}

			$BATCH_RESULT->addProcessedItem();
			if ($EXTRACT_CONTEXT->trace) {
				$BATCH_RESULT->printBatchDetail('RETS Listing: ' . $unique_value . $buffer);
			}else {
				if (!$quiet) {
					$BATCH_RESULT->printBatchNote();
				}
			}
		}

		// close database

		if ($TARGET_CONTEXT->target_type == 'RDB' ||
			$TARGET_CONTEXT->target_type == 'OR') {
			//$conn->Close();
		}

		// print errors

		if ($BATCH_RESULT->hasErrors()) {
			$BATCH_RESULT->printErrors();
		}

		// summary

		if ($EXTRACT_CONTEXT->trace) {
			if ($withDetail) {
				$BATCH_RESULT->printBatchSummary();
			}
		}

		return $BATCH_RESULT;
	}
	
  function write_rdb($LISTING_OBJECT, $SOURCE_CONTEXT, $TARGET_CONTEXT, $EXTRACT_CONTEXT, &$BATCH_RESULT, $METADATA_TABLE, $unique_value){
    //get access to the $mls object with data about this MLS feed
	  $mls = $this->mls;

	  //var to track whether we're updating or inserting for this particular listing.
  	$update = NULL;   
    
  	if ($TARGET_CONTEXT->include_images){
		  $imageInputFilter = $LISTING_OBJECT->getImageInputFilter();
			$sqlImagePath = NULL;			
			
			if ($imageInputFilter){
        $imageOutputFilter = $LISTING_OBJECT->getImageOutputFilter();
        $f_sql = NULL;
        $v_sql = NULL;
        $index = 0;
        foreach ($imageOutputFilter as $key => $val){
          if ($imageInputFilter[$index] != NO_VALUE_INDICATOR){
            // look if the input filter signifies a path
            if ($imageInputFilter[$index] == 'PATH'){
              $sqlImagePath = $val;
					  }

					  // store info in arrays 
            $f_sql[] = $val;
						$v_sql[] = $imageInputFilter[$index];
          }
				  ++$index;
        }
				$sqlImageFields = $f_sql;
				$sqlImageValues = $v_sql;
			}
		}

		// create a generic where clause
		$sqlWhere = ' WHERE ' . $LISTING_OBJECT->getUniqueColumn() . " = '%s' && field_mls_id_nid = '%s'";

		// check if listing exists
		$sql = 'SELECT * FROM {' . $TARGET_CONTEXT->data_table . '}' . $sqlWhere;
		$result = db_query($sql, $unique_value, $mls->nid);

		if(db_error()){
			$BATCH_RESULT->addError('duplicate get ERROR ' . $sql);
		}else{
			$duplicateRemoved = false;
      $clearToInsert = true;
			while($row = db_fetch_object($result)){
        if (!$EXTRACT_CONTEXT->refresh){
        	$clearToInsert = false;
        	continue;
        }

			  //rather than delete the row (see original), we can update it (in order preserve nid) - but we should delete the images 
			  $update = $row->nid;
        $this->existing_listings[] = $row->field_listing_number_value;
			  
        if ($TARGET_CONTEXT->include_images) {
          $detectWhere = ' WHERE ' . $TARGET_CONTEXT->image_table_key . '=' . "'%s' AND field_listing_id_nid = '%s'";
          $sql = 'SELECT ' . $sqlImagePath . ' FROM {' . $TARGET_CONTEXT->image_table . '}' . $detectWhere;

          $result2 = db_query($sql,$unique_value,$row->nid);
          if (db_error()) {
            $BATCH_RESULT->addError('duplicate select image ERROR ' . $sql);
          } else {
            //we don't want to have to download the photos again.
            $skip_photos = TRUE;
            /*while ($row2 = db_fetch_object($result2)) {
              $dup_image = $row2->$sqlImagePath;
              if (file_exists($dup_image)) {
                unlink($dup_image);
              }
            }*/
          }
  
          /*$sql = 'DELETE FROM {' . $TARGET_CONTEXT->image_table . '}' . $detectWhere;
          //$recordSet2 = $conn->Execute($sql);
          $result2 = db_query($sql,$unique_value,$row->nid);
          if (db_error()) {
            $BATCH_RESULT->addError('duplicate delete image ERROR ' . $sql);
          }*/
        }
        $duplicateRemoved = true;
			}
		}

		// insert into database
		$buffer = NULL;
		if (!$clearToInsert) {
		  $BATCH_RESULT->addSkippedItem();
      $buffer .= ' not downloaded, already exists in the database';
      return $buffer;
		}
		//fieldname wrapper depending on database type 
		switch ($TARGET_CONTEXT->brand){
      /*case 'mysql':
			  $wrapper_begin = '`';
				$wrapper_end = '`';
				break;*/
	
			case 'mssql':
				$wrapper_begin = '[';
				$wrapper_end = ']';
				break;

      default:
			  $wrapper_begin = NULL;
        $wrapper_end = NULL;
		}

		// prepare text data into lists
		//TODO can this be done more efficiently?  esp. the substr part
		$data = $LISTING_OBJECT->getData();
    $dataFields = array();
    $dataValues = array();
		foreach ($data as $key => $value) {
		  if ($value) {
		  	$dataFields[] = $key;
		  	$dataValues[] = $value;
			}
    }
		
		//attach mls specific information to the data fields/values
		$dataFields[] = 'field_mls_id_nid';
		$dataValues[] = $mls->nid;
		$dataFields[] = 'field_class_value';
		$dataValues[] = $SOURCE_CONTEXT->class_name;
		if(module_exists('green_franchise_listings')){
		  $dataFields[] = 'field_active_value';
		  $dataValues[] = 1;
		}
		
		//update or insert
		if ($update){
		  //we need to load the node, and then update the fields, and then save the node.
			$node = node_load($update);
			//TODO we may need to update a title?
		}else{
		  //get our user info, defaults to $uid = 1
      $node = $this->create_node('mls_listing',$mls->title . ' - ' . $unique_value);
    }
        
		//attach data fields and their values to the node
		$this->attach_field_names($node,$dataFields,$dataValues);
		$node->field_mls_id[0] = array('nid' => $this->mls->nid,'error_field' => 'field_mls_id][nids');
		$this->commit_node($node);   
    $this->new_listings[] = $node->field_listing_number[0]['value'];
		
		if($skip_photos == TRUE){
			$buffer .= ", skipping image downloads (already exists).";
			return $buffer;
		}
		// insert image data into database 
		$userImageSize = 0;
		if ($TARGET_CONTEXT->include_images) {
			
		  if ($TARGET_CONTEXT->image_reference_only) {
        $images = $LISTING_OBJECT->getImages();
				if ($images) {
					$index = NULL;
				  $multipartOffset = 1;
					foreach ($images as $key => $aURL) {	
						//prepare the values
						foreach ($sqlImageValues as $key2 => $val) {
							switch ($val) {
							  case 'ID':
									$imageValues[$key2] = $unique_value;
									break;

								case 'INDEX':
									$imageValues[$key2] = $key + $multipartOffset;
									$index = $imageValues[$key2];
									break;

								case 'URL':
									$imageValues[$key2] = $aURL;
									break;

								case 'PATH':
									$imageValues[$key2] = 'REFERENCE_ONLY';
									break;
	            }
						}

            $this->create_image_node($node,$sqlImageFields,$imageValues,$index);
					}
				}
			}else{
				if ($SOURCE_CONTEXT->media_location) {
					$images = $LISTING_OBJECT->getImages();
					if ($images != NULL) {
					  $multipartOffset = 1;
						$foundImage = true;
						foreach ($images as $key => $aURL) {
							//why are there two if clauses for foundImage?  TODO refractor
							if ($foundImage) {							
                $save_path = $this->createImagePath($TARGET_CONTEXT->image_download_path,
								$unique_value,
								$key + $multipartOffset);
								$foundImage = $this->downloadImage($save_path,
									$aURL,
									$EXTRACT_CONTEXT->trace,
									$BATCH_RESULT);
	              if ($foundImage) {
	              	$index = NULL;							
                  foreach ($sqlImageValues as $key2 => $val) {
									  $value = '';
										switch ($val) {
										  case 'ID':
											  $imageValues[$key2] = $unique_value;
												break;
	
											case 'INDEX':
											  $imageValues[$key2] = $key + $multipartOffset;
											  $index = $key + $multipartOffset;
												break;
	
											case 'URL':
												$imageValues[$key2] = $aURL;
												break;
	
											case 'PATH':
												$imageValues[$key2] = realpath($save_path);
												break;
										}
										$sql .= $value . "','";
									}
									
                  $this->create_image_node($node,$sqlImageFields,$imageValues,$index);
								}
							}
						}
						$userImageSize = sizeof($images);
					}
				}else {
				  $multipartOffset = 0;
					$mediaStart = 1;
					$mediaEnd = $EXTRACT_CONTEXT->max_images;
	
					// does this take advantage of multipart images?
					if ($SOURCE_CONTEXT->media_multipart && EXP_MULTIPART_CONTENT ) {
	          $images = $this->EXCHANGE->returnMediaObjects($SOURCE_CONTEXT->resource,$SOURCE_CONTEXT->media_type,$unique_value,false);
						$multipartOffset = 1;
						$mediaStart = 0;
						$mediaEnd = sizeof($images) - 1;
					}else {
						if ($SOURCE_CONTEXT->media_list_zero_start) {
						  $mediaStart = 0;
							--$mediaEnd;
						}
					}
	
					// get images
					for ($i = $mediaStart; $i <= $mediaEnd; $i++) {
					  if ($SOURCE_CONTEXT->media_multipart) {
		     			// assign media object
	  					$anImage = $images[$i];
						}else {
	  					// generate media object
							$anImage = $this->EXCHANGE->returnMediaObject($SOURCE_CONTEXT->resource,$SOURCE_CONTEXT->media_type,$unique_value,$i,false);
						}
	
						if (!$anImage) {
						  if ($this->EXCHANGE->getTransportTrace()) {
							  $this->EXCHANGE->trace('No images found in position $i, stopping search');
							}
						  break;
						}
						$save_path = $this->createImagePath($TARGET_CONTEXT->image_download_path,$unique_value,$i + $multipartOffset);
						if ($this->storeImage($save_path,$anImage,$EXTRACT_CONTEXT->trace,$BATCH_RESULT)) {
              $index = NULL;
							foreach ($sqlImageValues as $key2 => $val) {
					  		$value = '';
								switch ($val) {
						   		case 'ID':
								 	  $imageValues[$key2] = $unique_value;
									  break;
	
									case 'INDEX':
									  $imageValues[$key2] = $i + $multipartOffset;
									  $index = $imageValues[$key2];
										break;
	
									case 'URL':
									  $imageValues[$key2] = 'DIRECT';
										break;
	
									case 'PATH':
									  $imageValues[$key2] = realpath($save_path);
										break;
								}

							}
              
							$this->create_image_node($node,$sqlImageFields,$imageValues,$index);

							++$userImageSize;
						}
					}
				}
			}
		}
	
		// user notification
	
		if ($userImageSize == 1) {
		  $buffer .= ', 1 image downloaded.';
		}else {
		  $buffer .= ', ' . $userImageSize . ' images downloaded.';
		}
	
		if ($duplicateRemoved) {
		  $BATCH_RESULT->addDuplicatedItem();
			$BATCH_RESULT->addRefreshedItem();
		}else {
		  $BATCH_RESULT->addAdditionalItem();
		}
			
	  return $buffer;
	}
	
	
	function attach_field_names(&$node,$dataFields,$dataValues){
	    foreach($dataFields as $key => $dataField){
	    	//we have to strip out the last _value or _nid from the field name
	      if(substr($dataField,-4) == '_nid'){
          $field_name = substr($dataField,0,-4);
          $node->$field_name = array(array('nid' => $dataValues[$key],'error_field' => $field_name.'][nids'));
	      }
	      else{
	        if(substr($dataField,-6) == '_value'){
		        $field_name = substr($dataField,0,-6);
		      }else{
		      	$field_name = $dataField;
		      }
		      $node->$field_name = array(
		                             array(
		                               'value' => $dataValues[$key]
		                             )
		                           );
	      }
	    }

		}
  
	function create_node($type,$title){
    $user = $this->user;
    $user = user_load(array('uid'=>$user->uid));

    //establish our new node
    $node = new StdClass();
    $node->type = $type;
    $node->uid = $user->uid;
    $node->name = $user->name;
    $node->title = $title; 
    $node->body = '';
    $node->teaser = '';
    $node->status = 1;
    if(isset($this->mls->og_groups)){
    	foreach($this->mls->og_groups as $gid){
    		$node->og_groups[] = $gid;
    	}
    }
    return $node;
  }
  
  function commit_node(&$node){

  	//$node = node_submit($node);

    //module_invoke_all('submit', $node);
    node_save($node); //using node_save by itself seems to be the only thing that preserves the node_reference.
  	//module_invoke_all('nodeapi', $node, 'insert', NULL, NULL);
    
    cache_clear_all('content:'. $node->nid .':'. $node->vid, 'cache_content');
  }
  
  function create_image_node(&$listing_node,$sqlImageFields,$imageValues,$index=NULL){
    //add mls listing node id info 
    $sqlImageFields[] = 'field_listing_id_nid';
    $imageValues[] = $listing_node->nid;

    //create our image node
    $image_node = $this->create_node('mls_image',$listing_node->title . ' - Image ' . $index);

    //attach the field names
    $this->attach_field_names($image_node,$sqlImageFields,$imageValues);

    $this->commit_node($image_node);
  }

  function cleanup(){
     //remove nonexistant listings from db
    foreach($this->new_listings as $listing){
      if(!in_array($listing,$this->existing_listings)){
        $listings_to_remove[] = $listing;
      }
    }
    if($listings_to_remove){
      $remove = implode("','",$listings_to_remove);
      $result = db_query("SELECT nid FROM {content_type_mls_listing} WHERE field_listing_number_value IN ('$remove') AND field_mls_id_nid = %d",$this->mls->nid);
      while($row = db_fetch_object($result)){
        node_delete($row->nid);
        db_query("DELETE FROM {node_comment_statistics} WHERE nid = %s",$row->nid);
        
        //delete associated images
        $images = db_query("SELECT nid,field_path_value FROM {content_type_mls_image} WHERE field_listing_id_nid = %d",$row->nid);
        while($image_row = db_fetch_object($images)){
          node_delete($image_row->nid);
          db_query("DELETE FROM {node_comment_statistics} WHERE nid = %s",$image_row->nid);
          unlink($image_row->field_path_value);
        }
      }
    }
  }
}


/**
 * Classes from viele's run_interactive_job.php
 * 
 * These classes are directly from the aforementioned file.
 * They help format errors and statistics.
 */
class ProcessResults extends AbstractProcessResults{
     function ProcessResults()
     {
          parent::AbstractProcessResults();
     }

     function printErrors()
     {
          $buffer = NULL;
          $spacer = '  ';
          $buffer .= "\r\n";
          /*foreach ($this->errors as $key => $value) 
          {
               $buffer .= $spacer . $value . "\r\n";
          }*/
          flush_output($this->errors,'error');
     }

}

class Statistics extends AbstractStatistics{

     function Statistics($name)
     {
          parent::AbstractStatistics($name);
          $this->setSpacer('  ');
          $this->setEOL("\r\n");
     }

     function printErrors()
     {
          $buffer = NULL;
          $spacer = '  ';
          $buffer .= "\r\n";
          foreach ($this->errors as $key => $value) 
          {
               $buffer .= $spacer . $value . "\r\n";
          }
          flush_output($buffer);
     }

     function printBatchStart()
     {
          flush_output("\r\n" . $this->getStart() . "\r\n");
     }

     function printBatchDetail($detail)
     {
          $spacer = '  ';
          $buffer = NULL;
          $buffer .= $spacer;
          $buffer .= $detail;
          $buffer .= "\r\n";
          if ($this->hasErrors())
          {
               foreach ($this->errors as $key => $value) 
               {
                    $buffer .= $spacer . $value . "\r\n";
               }
          }
          flush_output($buffer);
     }

     function printBatchNote()
     {
          flush_output('.');
     }

     function printBatchSummary()
     {
          flush_output("\r\n" . $this->getSummary());
     }

     function getStart()
     {
          $this->markStart();
          return 'Starting ' . $this->name . 
                 ' at ' . date('F j, Y g:i:s A', $this->start) . 
                 "\r\n";
     }

     function getSummary($RETRIEVER = NULL)
     {

          $this->markFinish();
          $spacer = '  ';
          $buffer = $this->name . ' Completed at ' . 
                    date('F j, Y g:i:s A', $this->finish) .
                     "\r\n";

          $buffer .= "\r\n";

          $buffer .= 'Listings Processed: ' .
                     $this->getProcessed() .
                     "\r\n";

          $buffer .= $spacer .
                     'Additions: ' .
                     $this->getAdditions() .
                     "\r\n";

          $buffer .= $spacer .
                     'Duplicates found: ' .
                     $this->getDuplicates() .
                     "\r\n";

          $buffer .= $spacer .
                     $spacer .
                     'Refreshed: ' .
                     $this->getRefreshed() .
                     "\r\n";

          $buffer .= $spacer .
                     $spacer .
                     'Skipped: ' .
                     $this->getSkipped() .
                     "\r\n";

          $buffer .= "\r\n";

          $buffer .= 'Images Processed: ' .
                     $this->getImages() .
                     "\r\n";

          $buffer .= $spacer .
                     'Full: ' .
                     $this->getRaws() .
                     "\r\n";

          $buffer .= $spacer .
                     'Thumbnails: ' .
                     $this->getThumbs() .
                     "\r\n";

          $buffer .= $spacer .
                     'Missing or Corrupt: ' .
                     $this->getMissing() .
                     "\r\n";

          $buffer .= "\r\n";

          if ($this->start != NULL)
          {
               $buffer .= 'Performance ';
               $elapsed = $this->getRunTime();
               if ($elapsed == 0)
               {
                    $elapsed = 1;
               }
               $count = $this->getProcessed();
               $rate = round($count / $elapsed, 4);
               if ($count == 0)
               {
                    $speed = 0;
               }
               else
               {
                    $speed = round($elapsed / $count, 4);
               }
               $buffer .= $spacer .
                          'Total Elapsed Seconds: ' .
                          number_format($elapsed, 0) .
                          "\r\n";

               if ($RETRIEVER != NULL)
               {
                    $networkTime = $RETRIEVER->getNetworkTime();
                    $buffer .= $spacer .
                               $spacer .
                               'MLS Server: ' .
                               number_format($networkTime, 0) .
                               "\r\n";

                    $buffer .= $spacer .
                               $spacer .
                               'VieleRETS: ' .
                               number_format($elapsed = $networkTime, 0) .
                               "\r\n";
               }

               $buffer .= $spacer .
                          'Items per Second: ' .
                          number_format($rate, 4) .
                          "\r\n";

               $buffer .= $spacer .
                          'Seconds per Item: ' .
                          number_format($speed, 4) .
                          "\r\n";

          }

          return $buffer;
     }
}

function printBatchBanner()
{
//
// print tool information
//
     print('--------------------------------------------------' . "\r\n" .
           PROJECT_NAME . ' ' . PROJECT_VERSION . "\r\n" .
           reverse_htmlentities(PROJECT_COPYRIGHT) . "\r\n" .
           '--------------------------------------------------' . "\r\n" .
           "\r\n");
}

function collectInput($display = null,
                      $no_nulls = true)
{
     if ($display != null)
     {
          print($display);
     } 
     print('>'); 
     $input = trim(fgets(STDIN)); 
     if ($no_nulls)
     {
          if (strlen($input) == 0)
          {
              return 'CANCEL';
          }
     }
     return $input;
}

function displayList($title,
                     $list,
                     $keys_too = false)
{
     print($title . "\r\n"); 
     foreach ($list as $key => $value) 
     {
          $display = $value;
          if (is_array($display))
          {
               $display = null;
               foreach ($value as $key2 => $value2) 
               {
                    $display .= '[' . $value2 . '] ';
               }
          }
          else
          {
               if (strlen($display) == 0)
               {
                    $display = '[NO VALUE]';
               }
          }
          if ($keys_too)
          {
               print('  ' . $key . '=' . $display . "\r\n");
          }
          else
          {
               print('  ' . $display . "\r\n");
          }
     }
}

function reverse_htmlentities($mixed)
{
     $htmltable = Array();
     $htmltable['(C)'] = '&copy;';
     $htmltable['(R)'] = '&reg;';
//     $htmltable = get_html_translation_table(HTML_ENTITIES);
     foreach($htmltable as $key => $value)
     {
          $mixed = ereg_replace(addslashes($value),$key,$mixed);
     }
     return $mixed;
}

function flush_output($buffer,$type = 'status')
{
     /*print($buffer);
     flush();
     drupal_set_message($buffer);
     return $buffer;*/
	   foreach($buffer as $message){
	   	drupal_set_message($message,$type);
	   }
	   return '';
}

function printBatchErrors($partial,
                          $listingCount)
{
     if ($partial)
     {
          print(PARTIAL_RESULT_MESSAGE . "\r\n");
          //return PARTIAL_RESULT_MESSAGE . "\r\n";
     }
     if ($listingCount == 0)
     {
          print(NO_ITEMS_MESSAGE . "\r\n");
          //return NO_ITEMS_MESSAGE . "\r\n";
     }
}

function printBatchCompletion()
{
     print("\r\n" . 'Processing Completed' . "\r\n");
}

function postExecuteStatistics($S_CONFIGURATION,
                               $RETRIEVER)
{ 

//
// no stats
//

//$temp = $RETRIEVER->getNetworkTime();
//print("NetworkTime $temp\r\n");

}
?>